"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveEnv = exports.updateEnv = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
function updateEnv(userDir, _env) {
    return __awaiter(this, void 0, void 0, function* () {
        const env = yield (0, exports.resolveEnv)(_env);
        const ENV_DIR = path_1.default.join(userDir, "src", "env");
        const schema = path_1.default.join(ENV_DIR, "schema.ts");
        yield fs_extra_1.default.writeFile(schema, `import { z } from "zod";

export const serverScheme = z.object({\n${env.newServerScheme}
});

export const clientScheme = z.object({\n${env.newClientScheme}
});
`);
        if (env.newEnv.trim().length) {
            yield Promise.all([
                fs_extra_1.default.outputFile(path_1.default.join(userDir, ".env"), env.newEnv),
                fs_extra_1.default.outputFile(path_1.default.join(userDir, ".env.example"), env.newEnv),
            ]);
        }
    });
}
exports.updateEnv = updateEnv;
const resolveEnv = (env) => {
    return new Promise((resolve) => {
        var _a;
        let newEnv = "";
        let newClientScheme = "";
        let newServerScheme = "";
        let serverWasIn = false;
        let clientWasIn = false;
        for (const element of env) {
            const shouldAddNewLine = element.kind === "server" ? serverWasIn : clientWasIn;
            const value = `${shouldAddNewLine ? "\n" : ""}  ${element.key}: z.${element.type},`;
            if (element.kind === "server") {
                serverWasIn = true;
                newServerScheme += value;
            }
            else {
                clientWasIn = true;
                newClientScheme += value;
            }
            if (!element.ignore) {
                newEnv += `${element.key}=${(_a = element.defaulValue) !== null && _a !== void 0 ? _a : ""}\n`;
            }
        }
        return resolve({ newEnv, newClientScheme, newServerScheme });
    });
};
exports.resolveEnv = resolveEnv;
